@@ -6,12 +6,13 @@
 #include "RootAccessible.h"
 
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/PresShell.h"  // for nsAccUtils::GetDocAccessibleFor()
 
 #define CreateEvent CreateEventA
-#include "nsIDOMDocument.h"
 
 #include "Accessible-inl.h"
 #include "DocAccessible-inl.h"
+#include "mozilla/a11y/DocAccessibleParent.h"
 #include "nsAccessibilityService.h"
 #include "nsAccUtils.h"
 #include "nsCoreUtils.h"
@@ -20,21 +21,22 @@
 #include "Role.h"
 #include "States.h"
 #ifdef MOZ_XUL
-#include "XULTreeAccessible.h"
+#  include "XULTreeAccessible.h"
 #endif
 
+#include "mozilla/dom/BindingUtils.h"
+#include "mozilla/dom/CustomEvent.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/ScriptSettings.h"
+#include "mozilla/dom/BrowserHost.h"
 
-#include "nsIDocShellTreeItem.h"
 #include "nsIDocShellTreeOwner.h"
 #include "mozilla/dom/Event.h"
 #include "mozilla/dom/EventTarget.h"
-#include "nsIDOMCustomEvent.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
-#include "nsIDocument.h"
+#include "mozilla/dom/Document.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIPropertyBag2.h"
-#include "nsIServiceManager.h"
 #include "nsPIDOMWindow.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsReadableUtils.h"
@@ -42,7 +44,7 @@
 #include "nsGlobalWindow.h"
 
 #ifdef MOZ_XUL
-#include "nsIXULWindow.h"
+#  include "nsIAppWindow.h"
 #endif
 
 using namespace mozilla;
@@ -57,7 +59,7 @@ NS_IMPL_ISUPPORTS_INHERITED(RootAccessible, DocAccessible, nsIDOMEventListener)
 ////////////////////////////////////////////////////////////////////////////////
 // Constructor/destructor
 
-RootAccessible::RootAccessible(nsIDocument* aDocument, nsIPresShell* aPresShell)
+RootAccessible::RootAccessible(Document* aDocument, PresShell* aPresShell)
     : DocAccessibleWrap(aDocument, aPresShell) {
   mType = eRootType;
 }
@@ -67,7 +69,7 @@ RootAccessible::~RootAccessible() {}
 ////////////////////////////////////////////////////////////////////////////////
 // Accessible
 
-ENameValueFlag RootAccessible::Name(nsString& aName) {
+ENameValueFlag RootAccessible::Name(nsString& aName) const {
   aName.Truncate();
 
   if (ARIARoleMap()) {
@@ -79,38 +81,28 @@ ENameValueFlag RootAccessible::Name(nsString& aName) {
   return eNameOK;
 }
 
-role RootAccessible::NativeRole() {
-  // If it's a <dialog> or <wizard>, use roles::DIALOG instead
-  dom::Element* rootElm = mDocumentNode->GetRootElement();
-  if (rootElm &&
-      rootElm->IsAnyOfXULElements(nsGkAtoms::dialog, nsGkAtoms::wizard))
-    return roles::DIALOG;
-
-  return DocAccessibleWrap::NativeRole();
-}
-
 // RootAccessible protected member
 #ifdef MOZ_XUL
-uint32_t RootAccessible::GetChromeFlags() {
+uint32_t RootAccessible::GetChromeFlags() const {
   // Return the flag set for the top level window as defined
   // by nsIWebBrowserChrome::CHROME_WINDOW_[FLAGNAME]
-  // Not simple: nsIXULWindow is not just a QI from nsIDOMWindow
+  // Not simple: nsIAppWindow is not just a QI from nsIDOMWindow
   nsCOMPtr<nsIDocShell> docShell = nsCoreUtils::GetDocShellFor(mDocumentNode);
   NS_ENSURE_TRUE(docShell, 0);
   nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
   docShell->GetTreeOwner(getter_AddRefs(treeOwner));
   NS_ENSURE_TRUE(treeOwner, 0);
-  nsCOMPtr<nsIXULWindow> xulWin(do_GetInterface(treeOwner));
-  if (!xulWin) {
+  nsCOMPtr<nsIAppWindow> appWin(do_GetInterface(treeOwner));
+  if (!appWin) {
     return 0;
   }
   uint32_t chromeFlags;
-  xulWin->GetChromeFlags(&chromeFlags);
+  appWin->GetChromeFlags(&chromeFlags);
   return chromeFlags;
 }
 #endif
 
-uint64_t RootAccessible::NativeState() {
+uint64_t RootAccessible::NativeState() const {
   uint64_t state = DocAccessibleWrap::NativeState();
   if (state & states::DEFUNCT) return state;
 
@@ -149,25 +141,29 @@ const char* const kEventTypes[] = {
     // add ourself as a CheckboxStateChange listener (custom event fired in
     // HTMLInputElement.cpp)
     "CheckboxStateChange",
-    // add ourself as a RadioStateChange Listener ( custom event fired in in
-    // HTMLInputElement.cpp  & radio.xml)
+    // add ourself as a RadioStateChange Listener (custom event fired in in
+    // HTMLInputElement.cpp & radio.js)
     "RadioStateChange", "popupshown", "popuphiding", "DOMMenuInactive",
     "DOMMenuItemActive", "DOMMenuItemInactive", "DOMMenuBarActive",
-    "DOMMenuBarInactive"};
+    "DOMMenuBarInactive", "scroll"};
 
 nsresult RootAccessible::AddEventListeners() {
   // EventTarget interface allows to register event listeners to
   // receive untrusted events (synthetic events generated by untrusted code).
   // For example, XBL bindings implementations for elements that are hosted in
   // non chrome document fire untrusted events.
-  nsCOMPtr<EventTarget> nstarget = mDocumentNode;
+  // We must use the window's parent target in order to receive events from
+  // iframes and shadow DOM; e.g. ValueChange events from a <select> in an
+  // iframe or shadow DOM. The root document itself doesn't receive these.
+  nsPIDOMWindowOuter* window = mDocumentNode->GetWindow();
+  nsCOMPtr<EventTarget> nstarget = window ? window->GetParentTarget() : nullptr;
 
   if (nstarget) {
     for (const char *const *e = kEventTypes, *const *e_end =
                                                  ArrayEnd(kEventTypes);
          e < e_end; ++e) {
       nsresult rv = nstarget->AddEventListener(NS_ConvertASCIItoUTF16(*e), this,
-                                               true, true, 2);
+                                               true, true);
       NS_ENSURE_SUCCESS(rv, rv);
     }
   }
@@ -176,14 +172,13 @@ nsresult RootAccessible::AddEventListeners() {
 }
 
 nsresult RootAccessible::RemoveEventListeners() {
-  nsCOMPtr<EventTarget> target = mDocumentNode;
+  nsPIDOMWindowOuter* window = mDocumentNode->GetWindow();
+  nsCOMPtr<EventTarget> target = window ? window->GetParentTarget() : nullptr;
   if (target) {
     for (const char *const *e = kEventTypes, *const *e_end =
                                                  ArrayEnd(kEventTypes);
          e < e_end; ++e) {
-      nsresult rv =
-          target->RemoveEventListener(NS_ConvertASCIItoUTF16(*e), this, true);
-      NS_ENSURE_SUCCESS(rv, rv);
+      target->RemoveEventListener(NS_ConvertASCIItoUTF16(*e), this, true);
     }
   }
 
@@ -202,11 +197,18 @@ void RootAccessible::DocumentActivated(DocAccessible* aDocument) {}
 // nsIDOMEventListener
 
 NS_IMETHODIMP
-RootAccessible::HandleEvent(nsIDOMEvent* aDOMEvent) {
+RootAccessible::HandleEvent(Event* aDOMEvent) {
   MOZ_ASSERT(aDOMEvent);
-  Event* event = aDOMEvent->InternalDOMEvent();
+  if (IsDefunct()) {
+    // Even though we've been shut down, RemoveEventListeners might not have
+    // removed the event handlers on the window's parent target if GetWindow
+    // returned null, so we might still get events here in this case. We should
+    // just ignore these events.
+    return NS_OK;
+  }
+
   nsCOMPtr<nsINode> origTargetNode =
-      do_QueryInterface(event->GetOriginalTarget());
+      do_QueryInterface(aDOMEvent->GetOriginalTarget());
   if (!origTargetNode) return NS_OK;
 
 #ifdef A11Y_LOG
@@ -221,45 +223,54 @@ RootAccessible::HandleEvent(nsIDOMEvent* aDOMEvent) {
       GetAccService()->GetDocAccessible(origTargetNode->OwnerDoc());
 
   if (document) {
-    // Root accessible exists longer than any of its descendant documents so
-    // that we are guaranteed notification is processed before root accessible
-    // is destroyed.
-    document->HandleNotification<RootAccessible, nsIDOMEvent>(
-        this, &RootAccessible::ProcessDOMEvent, aDOMEvent);
+    nsAutoString eventType;
+    aDOMEvent->GetType(eventType);
+    if (eventType.EqualsLiteral("scroll")) {
+      // We don't put this in the notification queue for 2 reasons:
+      // 1. We will flood the queue with repetitive events.
+      // 2. Since this doesn't necessarily touch layout, we are not
+      //    guaranteed to have a WillRefresh tick any time soon.
+      document->HandleScroll(origTargetNode);
+    } else {
+      // Root accessible exists longer than any of its descendant documents so
+      // that we are guaranteed notification is processed before root accessible
+      // is destroyed.
+      // For shadow DOM, GetOriginalTarget on the Event returns null if we
+      // process the event async, so we must pass the target node as well.
+      document->HandleNotification<RootAccessible, Event, nsINode>(
+          this, &RootAccessible::ProcessDOMEvent, aDOMEvent, origTargetNode);
+    }
   }
 
   return NS_OK;
 }
 
 // RootAccessible protected
-void RootAccessible::ProcessDOMEvent(nsIDOMEvent* aDOMEvent) {
+void RootAccessible::ProcessDOMEvent(Event* aDOMEvent, nsINode* aTarget) {
   MOZ_ASSERT(aDOMEvent);
-  Event* event = aDOMEvent->InternalDOMEvent();
-  nsCOMPtr<nsINode> origTargetNode =
-      do_QueryInterface(event->GetOriginalTarget());
+  MOZ_ASSERT(aTarget);
 
   nsAutoString eventType;
   aDOMEvent->GetType(eventType);
 
 #ifdef A11Y_LOG
   if (logging::IsEnabled(logging::eDOMEvents))
-    logging::DOMEvent("processed", origTargetNode, eventType);
+    logging::DOMEvent("processed", aTarget, eventType);
 #endif
 
   if (eventType.EqualsLiteral("popuphiding")) {
-    HandlePopupHidingEvent(origTargetNode);
+    HandlePopupHidingEvent(aTarget);
     return;
   }
 
   DocAccessible* targetDocument =
-      GetAccService()->GetDocAccessible(origTargetNode->OwnerDoc());
+      GetAccService()->GetDocAccessible(aTarget->OwnerDoc());
   if (!targetDocument) {
     // Document has ceased to exist.
     return;
   }
 
-  Accessible* accessible =
-      targetDocument->GetAccessibleOrContainer(origTargetNode);
+  Accessible* accessible = targetDocument->GetAccessibleOrContainer(aTarget);
   if (!accessible) return;
 
 #ifdef MOZ_XUL
@@ -335,7 +346,16 @@ void RootAccessible::ProcessDOMEvent(nsIDOMEvent* aDOMEvent) {
     // If multiselect tree, we should fire selectionadd or selection removed
     if (FocusMgr()->HasDOMFocus(targetNode)) {
       nsCOMPtr<nsIDOMXULMultiSelectControlElement> multiSel =
-          do_QueryInterface(targetNode);
+          targetNode->AsElement()->AsXULMultiSelectControl();
+      if (!multiSel) {
+        // This shouldn't be possible. All XUL trees should have
+        // nsIDOMXULMultiSelectControlElement, and the tree is focused, so it
+        // shouldn't be dying. Nevertheless, this sometimes happens in the wild
+        // (bug 1597043).
+        MOZ_ASSERT_UNREACHABLE(
+            "XUL tree doesn't have nsIDOMXULMultiSelectControlElement");
+        return;
+      }
       nsAutoString selType;
       multiSel->GetSelType(selType);
       if (selType.IsEmpty() || !selType.EqualsLiteral("single")) {
@@ -433,12 +453,13 @@ void RootAccessible::ProcessDOMEvent(nsIDOMEvent* aDOMEvent) {
 
 void RootAccessible::Shutdown() {
   // Called manually or by Accessible::LastRelease()
-  if (!PresShell()) return;  // Already shutdown
-
+  if (HasShutdown()) {
+    return;
+  }
   DocAccessibleWrap::Shutdown();
 }
 
-Relation RootAccessible::RelationByType(RelationType aType) {
+Relation RootAccessible::RelationByType(RelationType aType) const {
   if (!mDocumentNode || aType != RelationType::EMBEDS)
     return DocAccessibleWrap::RelationByType(aType);
 
@@ -446,7 +467,7 @@ Relation RootAccessible::RelationByType(RelationType aType) {
     nsCOMPtr<nsPIDOMWindowOuter> contentWindow =
         nsGlobalWindowOuter::Cast(rootWindow)->GetContent();
     if (contentWindow) {
-      nsCOMPtr<nsIDocument> contentDocumentNode = contentWindow->GetDoc();
+      RefPtr<Document> contentDocumentNode = contentWindow->GetDoc();
       if (contentDocumentNode) {
         DocAccessible* contentDocument =
             GetAccService()->GetDocAccessible(contentDocumentNode);
@@ -490,6 +511,25 @@ void RootAccessible::HandlePopupShownEvent(Accessible* aAccessible) {
           new AccStateChangeEvent(combobox, states::EXPANDED, true);
       if (event) nsEventShell::FireEvent(event);
     }
+
+    // If aria-activedescendant is present, redirect focus.
+    // This is needed for parent process <select> dropdowns, which use a
+    // menulist containing div elements instead of XUL menuitems. XUL menuitems
+    // fire DOMMenuItemActive events from layout instead.
+    MOZ_ASSERT(aAccessible->Elm());
+    if (aAccessible->Elm()->HasAttr(kNameSpaceID_None,
+                                    nsGkAtoms::aria_activedescendant)) {
+      Accessible* activeDescendant = aAccessible->CurrentItem();
+      if (activeDescendant) {
+        FocusMgr()->ActiveItemChanged(activeDescendant, false);
+#ifdef A11Y_LOG
+        if (logging::IsEnabled(logging::eFocus)) {
+          logging::ActiveItemChangeCausedBy("ARIA activedescendant on popup",
+                                            activeDescendant);
+        }
+#endif
+      }
+    }
   }
 }
 
@@ -594,18 +634,35 @@ void RootAccessible::HandlePopupHidingEvent(nsINode* aPopupNode) {
 }
 
 #ifdef MOZ_XUL
-void RootAccessible::HandleTreeRowCountChangedEvent(
-    nsIDOMEvent* aEvent, XULTreeAccessible* aAccessible) {
-  nsCOMPtr<nsIDOMCustomEvent> customEvent(do_QueryInterface(aEvent));
+static void GetPropertyBagFromEvent(Event* aEvent,
+                                    nsIPropertyBag2** aPropertyBag) {
+  *aPropertyBag = nullptr;
+
+  CustomEvent* customEvent = aEvent->AsCustomEvent();
   if (!customEvent) return;
 
-  nsCOMPtr<nsIVariant> detailVariant;
-  customEvent->GetDetail(getter_AddRefs(detailVariant));
-  if (!detailVariant) return;
+  AutoJSAPI jsapi;
+  if (!jsapi.Init(customEvent->GetParentObject())) return;
 
-  nsCOMPtr<nsISupports> supports;
-  detailVariant->GetAsISupports(getter_AddRefs(supports));
-  nsCOMPtr<nsIPropertyBag2> propBag(do_QueryInterface(supports));
+  JSContext* cx = jsapi.cx();
+  JS::Rooted<JS::Value> detail(cx);
+  customEvent->GetDetail(cx, &detail);
+  if (!detail.isObject()) return;
+
+  JS::Rooted<JSObject*> detailObj(cx, &detail.toObject());
+
+  nsresult rv;
+  nsCOMPtr<nsIPropertyBag2> propBag;
+  rv = UnwrapArg<nsIPropertyBag2>(cx, detailObj, getter_AddRefs(propBag));
+  if (NS_FAILED(rv)) return;
+
+  propBag.forget(aPropertyBag);
+}
+
+void RootAccessible::HandleTreeRowCountChangedEvent(
+    Event* aEvent, XULTreeAccessible* aAccessible) {
+  nsCOMPtr<nsIPropertyBag2> propBag;
+  GetPropertyBagFromEvent(aEvent, getter_AddRefs(propBag));
   if (!propBag) return;
 
   nsresult rv;
@@ -620,17 +677,9 @@ void RootAccessible::HandleTreeRowCountChangedEvent(
 }
 
 void RootAccessible::HandleTreeInvalidatedEvent(
-    nsIDOMEvent* aEvent, XULTreeAccessible* aAccessible) {
-  nsCOMPtr<nsIDOMCustomEvent> customEvent(do_QueryInterface(aEvent));
-  if (!customEvent) return;
-
-  nsCOMPtr<nsIVariant> detailVariant;
-  customEvent->GetDetail(getter_AddRefs(detailVariant));
-  if (!detailVariant) return;
-
-  nsCOMPtr<nsISupports> supports;
-  detailVariant->GetAsISupports(getter_AddRefs(supports));
-  nsCOMPtr<nsIPropertyBag2> propBag(do_QueryInterface(supports));
+    Event* aEvent, XULTreeAccessible* aAccessible) {
+  nsCOMPtr<nsIPropertyBag2> propBag;
+  GetPropertyBagFromEvent(aEvent, getter_AddRefs(propBag));
   if (!propBag) return;
 
   int32_t startRow = 0, endRow = -1, startCol = 0, endCol = -1;
@@ -648,12 +697,12 @@ ProxyAccessible* RootAccessible::GetPrimaryRemoteTopLevelContentDoc() const {
   mDocumentNode->GetDocShell()->GetTreeOwner(getter_AddRefs(owner));
   NS_ENSURE_TRUE(owner, nullptr);
 
-  nsCOMPtr<nsITabParent> tabParent;
-  owner->GetPrimaryTabParent(getter_AddRefs(tabParent));
-  if (!tabParent) {
+  nsCOMPtr<nsIRemoteTab> remoteTab;
+  owner->GetPrimaryRemoteTab(getter_AddRefs(remoteTab));
+  if (!remoteTab) {
     return nullptr;
   }
 
-  auto tab = static_cast<dom::TabParent*>(tabParent.get());
+  auto tab = static_cast<dom::BrowserHost*>(remoteTab.get());
   return tab->GetTopLevelDocAccessible();
 }
