@@ -2,18 +2,20 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* exported switchToView, getSelectedDay, scheduleMidnightUpdate,
+/* exported switchToView, getSelectedDay, scheduleMidnightUpdate, minimonthPick,
  *          observeViewDaySelect, toggleOrientation,
  *          toggleWorkdaysOnly, toggleTasksInView, toggleShowCompletedInView,
- *          goToDate, getLastCalendarView, deleteSelectedEvents,
+ *          goToDate, gLastShownCalendarView, deleteSelectedEvents,
  *          editSelectedEvents, selectAllEvents, calendarNavigationBar
  */
 
-/* import-globals-from calendar-chrome-startup.js */
 /* import-globals-from calendar-item-editing.js */
+/* global gCurrentMode */
 
-var { cal } = ChromeUtils.import("resource://calendar/modules/calUtils.jsm");
-var { countOccurrences } = ChromeUtils.import("resource://calendar/modules/calRecurrenceUtils.jsm");
+var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
+var { countOccurrences } = ChromeUtils.import(
+  "resource:///modules/calendar/calRecurrenceUtils.jsm"
+);
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 /**
@@ -27,12 +29,11 @@ var calendarViewController = {
    * Creates a new event
    * @see calICalendarViewController
    */
-  createNewEvent: function(calendar, startTime, endTime, forceAllday) {
+  createNewEvent(calendar, startTime, endTime, forceAllday) {
     // if we're given both times, skip the dialog
     if (startTime && endTime && !startTime.isDate && !endTime.isDate) {
       let item = cal.createEvent();
       setDefaultItemValues(item, calendar, startTime, endTime);
-      item.title = cal.l10n.getCalString("newEvent");
       doTransaction("add", item, item.calendar, null, null);
     } else {
       createEventWithDialog(calendar, startTime, null, null, null, forceAllday);
@@ -43,7 +44,7 @@ var calendarViewController = {
    * Modifies the given occurrence
    * @see calICalendarViewController
    */
-  modifyOccurrence: function(occurrence, newStartTime, newEndTime, newTitle) {
+  modifyOccurrence(occurrence, newStartTime, newEndTime, newTitle) {
     // if modifying this item directly (e.g. just dragged to new time),
     // then do so; otherwise pop up the dialog
     if (newStartTime || newEndTime || newTitle) {
@@ -88,13 +89,7 @@ var calendarViewController = {
    * Deletes the given occurrences
    * @see calICalendarViewController
    */
-  deleteOccurrences: function(
-    count,
-    occurrencesArg,
-    useParentItems,
-    doNotConfirm,
-    extResponseArg = null
-  ) {
+  deleteOccurrences(occurrencesArg, useParentItems, doNotConfirm, extResponseArg = null) {
     startBatchTransaction();
     let recurringItems = {};
     let extResponse = extResponseArg || { responseMode: Ci.calIItipItem.USER };
@@ -202,7 +197,7 @@ function switchToView(viewType) {
   let currentSelection = [];
 
   // Set up the view commands
-  let views = viewDeck.childNodes;
+  let views = viewDeck.children;
   for (let i = 0; i < views.length; i++) {
     let view = views[i];
     let commandId = "calendar_" + view.id + "_command";
@@ -252,7 +247,7 @@ function switchToView(viewType) {
 
   try {
     selectedDay = viewDeck.selectedPanel.selectedDay;
-    currentSelection = viewDeck.selectedPanel.getSelectedItems({});
+    currentSelection = viewDeck.selectedPanel.getSelectedItems();
   } catch (ex) {
     // This dies if no view has even been chosen this session, but that's
     // ok because we'll just use cal.dtz.now() below.
@@ -278,9 +273,9 @@ function switchToView(viewType) {
   }
 
   view.goToDay(selectedDay);
-  view.setSelectedItems(currentSelection.length, currentSelection);
+  view.setSelectedItems(currentSelection);
 
-  onCalendarViewResize();
+  view.onResize(view);
 }
 
 /**
@@ -331,7 +326,7 @@ function scheduleMidnightUpdate(refreshCallback) {
   // workaround bug 291386.  If we don't, we stand a decent chance of getting
   // stuck in an infinite loop.
   let udCallback = {
-    notify: function(timer) {
+    notify(timer) {
       refreshCallback();
     },
   };
@@ -405,17 +400,27 @@ function observeViewDaySelect(event) {
     jsMainDate = new Date(mainDate.year, mainDate.month, mainDate.day);
   }
 
-  // The minimonth binding might not be ready yet, because it's XBL
-  // and XBL is hopeless. Wait for it if necessary.
-  let minimonth = getMinimonth();
-  if ("selectDate" in minimonth) {
-    minimonth.selectDate(jsDate, jsMainDate);
-  } else {
-    minimonth.onBindingReady = () => minimonth.selectDate(jsDate, jsMainDate);
-  }
+  getMinimonth().selectDate(jsDate, jsMainDate);
   currentView().focus();
 }
 
+/**
+ * Shows the given date in the current view, if in calendar mode.
+ *
+ * @param aNewDate      The new date as a JSDate.
+ */
+function minimonthPick(aNewDate) {
+  if (gCurrentMode == "calendar" || gCurrentMode == "task") {
+    let cdt = cal.dtz.jsDateToDateTime(aNewDate, currentView().timezone);
+    cdt.isDate = true;
+    currentView().goToDay(cdt);
+
+    // update date filter for task tree
+    let tree = document.getElementById("calendar-task-tree");
+    tree.updateFilter();
+  }
+}
+
 /**
  * Provides a neutral way to get the minimonth, regardless of whether we're in
  * Sunbird or Lightning.
@@ -435,7 +440,7 @@ function toggleOrientation() {
   cmd.setAttribute("checked", newValue);
 
   let deck = getViewDeck();
-  for (let view of deck.childNodes) {
+  for (let view of deck.children) {
     view.rotated = newValue == "true";
   }
 
@@ -454,7 +459,7 @@ function toggleWorkdaysOnly() {
   cmd.setAttribute("checked", newValue);
 
   let deck = getViewDeck();
-  for (let view of deck.childNodes) {
+  for (let view of deck.children) {
     view.workdaysOnly = newValue == "true";
   }
 
@@ -471,7 +476,7 @@ function toggleTasksInView() {
   cmd.setAttribute("checked", newValue);
 
   let deck = getViewDeck();
-  for (let view of deck.childNodes) {
+  for (let view of deck.children) {
     view.tasksInView = newValue == "true";
   }
 
@@ -488,7 +493,7 @@ function toggleShowCompletedInView() {
   cmd.setAttribute("checked", newValue);
 
   let deck = getViewDeck();
-  for (let view of deck.childNodes) {
+  for (let view of deck.children) {
     view.showCompleted = newValue == "true";
   }
 
@@ -506,43 +511,54 @@ function goToDate(date) {
   currentView().goToDay(date);
 }
 
-/**
- * Returns the calendar view that was selected before restart, or the current
- * calendar view if it has already been set in this session
- *
- * @return          The last calendar view.
- */
-function getLastCalendarView() {
-  if (Services.xulStore.hasValue(document.location.href, "view-deck", "selectedIndex")) {
-    let deck = getViewDeck();
-    let selectedIndex = Services.xulStore.getValue(
-      document.location.href,
-      "view-deck",
-      "selectedIndex"
-    );
-    let viewNode = deck.childNodes[selectedIndex];
-    return viewNode.id.replace(/-view/, "");
-  }
+var gLastShownCalendarView = {
+  _lastView: null,
 
-  // No deck item was selected beforehand, default to week view.
-  return "week";
-}
+  /**
+   * Returns the calendar view that was selected before restart, or the current
+   * calendar view if it has already been set in this session.
+   *
+   * @return {string} The last calendar view.
+   */
+  get() {
+    if (!this._lastView) {
+      if (Services.xulStore.hasValue(document.location.href, "view-deck", "selectedIndex")) {
+        let deck = getViewDeck();
+        let selectedIndex = Services.xulStore.getValue(
+          document.location.href,
+          "view-deck",
+          "selectedIndex"
+        );
+        let viewNode = deck.children[selectedIndex];
+        this._lastView = viewNode.id.replace(/-view/, "");
+      } else {
+        // No deck item was selected beforehand, default to week view.
+        this._lastView = "week";
+      }
+    }
+    return this._lastView;
+  },
+
+  set(view) {
+    this._lastView = view;
+  },
+};
 
 /**
  * Deletes items currently selected in the view and clears selection.
  */
 function deleteSelectedEvents() {
-  let selectedItems = currentView().getSelectedItems({});
-  calendarViewController.deleteOccurrences(selectedItems.length, selectedItems, false, false);
+  let selectedItems = currentView().getSelectedItems();
+  calendarViewController.deleteOccurrences(selectedItems, false, false);
   // clear selection
-  currentView().setSelectedItems(0, [], true);
+  currentView().setSelectedItems([], true);
 }
 
 /**
  * Edit the items currently selected in the view with the event dialog.
  */
 function editSelectedEvents() {
-  let selectedItems = currentView().getSelectedItems({});
+  let selectedItems = currentView().getSelectedItems();
   if (selectedItems && selectedItems.length >= 1) {
     modifyEventWithDialog(selectedItems[0], null, true);
   }
@@ -555,10 +571,10 @@ function selectAllEvents() {
   let items = [];
   let listener = {
     QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
-    onOperationComplete: function(calendar, status, operationType, id, detail) {
-      currentView().setSelectedItems(items.length, items, false);
+    onOperationComplete(calendar, status, operationType, id, detail) {
+      currentView().setSelectedItems(items, false);
     },
-    onGetResult: function(calendar, status, itemType, detail, count, itemsArg) {
+    onGetResult(calendar, status, itemType, detail, itemsArg) {
       for (let item of itemsArg) {
         items.push(item);
       }
@@ -587,7 +603,7 @@ function selectAllEvents() {
 }
 
 var calendarNavigationBar = {
-  setDateRange: function(startDate, endDate) {
+  setDateRange(startDate, endDate) {
     let docTitle = "";
     if (startDate) {
       let intervalLabel = document.getElementById("intervalDescription");
@@ -595,7 +611,7 @@ var calendarNavigationBar = {
       let secondWeekNo = firstWeekNo;
       let weekLabel = document.getElementById("calendarWeek");
       if (startDate.nativeTime == endDate.nativeTime) {
-        intervalLabel.value = cal.getDateFormatter().formatDate(startDate);
+        intervalLabel.value = cal.dtz.formatter.formatDate(startDate);
       } else {
         intervalLabel.value = currentView().getRangeDescription();
         secondWeekNo = cal.getWeekInfoService().getWeekTitle(endDate);
@@ -615,7 +631,7 @@ var calendarNavigationBar = {
       }
       docTitle = intervalLabel.value;
     }
-    if (document.getElementById("modeBroadcaster").getAttribute("mode") == "calendar") {
+    if (gCurrentMode == "calendar") {
       document.title =
         (docTitle ? docTitle + " - " : "") +
         cal.l10n.getAnyString("branding", "brand", "brandFullName");
@@ -630,13 +646,13 @@ var calendarNavigationBar = {
  */
 var timeIndicator = {
   timer: null,
-  start: function(interval, thisArg) {
+  start(interval, thisArg) {
     timeIndicator.timer = setInterval(
       () => thisArg.updateTimeIndicatorPosition(false),
       interval * 1000
     );
   },
-  cancel: function() {
+  cancel() {
     if (timeIndicator.timer) {
       clearTimeout(timeIndicator.timer);
       timeIndicator.timer = null;
@@ -646,7 +662,7 @@ var timeIndicator = {
 };
 
 var timezoneObserver = {
-  observe: function() {
+  observe() {
     let minimonth = getMinimonth();
     minimonth.update(minimonth.value);
   },
